
# q-expansion of newform 336.4.q.l, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 336
weight = 4
poly_data = [189, -747, 802, -111, 58, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[213, -331, -81, -43, -3, -1], 66], [[3, -28, 29, -2, 1, 0], 6], [[723, -1366, 157, -108, 5, -2], 132], [[-219, 40, -41, 2, -1, 0], 12], [[-10203, 23415, -2249, 1687, -67, 29], 132]]

hecke_ring_character_values = [[127, [1, 0, 0, 0, 0, 0]], [85, [1, 0, 0, 0, 0, 0]], [113, [1, 0, 0, 0, 0, 0]], [241, [0, 0, 0, -1, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0], [3, 0, 0, -3, 0, 0], [0, 0, 0, 4, -1, -1], [2, -1, -1, -3, -1, 0], [-6, 1, 1, 6, 0, 1], [-7, 0, 3, 0, -1, 0], [34, 1, 1, -34, 0, -2], [0, 8, 0, 69, -5, -5], [0, 7, 0, -94, 2, 2], [-28, 0, 5, 0, 11, 0], [-39, 1, 1, 39, 0, 14], [0, 3, 0, 113, -13, -13], [-166, 0, 0, 0, -18, 0], [-13, 0, -20, 0, 3, 0], [0, -11, 0, -36, -18, -18], [134, -4, -4, -134, 0, 17], [94, 16, 16, -94, 0, -3], [0, 20, 0, 22, -32, -32], [-25, -6, -6, 25, 0, 25], [-16, 0, 51, 0, 14, 0], [47, 39, 39, -47, 0, 33], [0, -15, 0, -821, 0, 0], [32, 0, 40, 0, 19, 0], [0, -24, 0, 60, -70, -70], [-982, 0, -13, 0, 5, 0], [78, -28, -28, -78, 0, -16], [0, -20, 0, 481, 23, 23], [0, -8, 0, -898, 9, 9], [567, 23, 23, -567, 0, 13], [188, 0, 39, 0, 68, 0], [571, 0, 11, 0, 100, 0], [0, -23, 0, -302, 91, 91], [1388, -46, -46, -1388, 0, -46], [-521, 0, -118, 0, -97, 0], [0, -114, 0, 636, 62, 62], [-460, -49, -49, 460, 0, 97], [-618, -78, -78, 618, 0, 66], [0, 86, 0, -800, -52, -52], [368, 0, 111, 0, 206, 0], [0, -117, 0, -2034, 28, 28], [-2752, -21, -21, 2752, 0, 140], [-287, 0, -63, 0, -49, 0], [0, -211, 0, -2568, 120, 120], [-2367, -104, -104, 2367, 0, 22], [2158, 0, -71, 0, -108, 0], [2100, 102, 102, -2100, 0, -88], [2092, 0, 34, 0, 2, 0], [-932, 0, 295, 0, 99, 0], [770, 85, 85, -770, 0, -213], [0, 161, 0, -3225, 61, 61], [0, -107, 0, -660, -184, -184], [1238, 0, 274, 0, 86, 0], [1198, -121, -121, -1198, 0, 237], [2786, 0, -286, 0, -211, 0], [0, 275, 0, -2488, 166, 166], [-2152, 32, 32, 2152, 0, 54], [1326, 19, 19, -1326, 0, 49], [0, 21, 0, -1452, 19, 19], [-2965, -147, -147, 2965, 0, 483], [1354, 0, 349, 0, -216, 0], [3289, 66, 66, -3289, 0, 295], [-4376, 0, -359, 0, 303, 0], [-4815, 0, 58, 0, -285, 0], [-4586, -130, -130, 4586, 0, -50], [0, -98, 0, -4967, -50, -50], [0, 233, 0, 2248, 273, 273], [0, 160, 0, 499, 249, 249], [1729, 0, -226, 0, 72, 0], [-4846, 65, 65, 4846, 0, 126], [-698, 0, 502, 0, 460, 0], [5408, 3, 3, -5408, 0, 208], [0, 560, 0, 2920, -102, -102], [-3213, -103, -103, 3213, 0, -230], [0, -285, 0, -787, 163, 163], [2609, 0, -44, 0, -185, 0], [0, -49, 0, -10502, -180, -180], [-336, -599, -599, 336, 0, 398], [0, -145, 0, -2235, -561, -561], [0, 339, 0, -3282, -318, -318], [3443, 42, 42, -3443, 0, -398], [-3366, 0, 632, 0, 182, 0], [-3027, 0, 479, 0, 537, 0], [-4120, -107, -107, 4120, 0, 284], [13103, 0, -71, 0, -153, 0], [0, -395, 0, 5780, -501, -501], [0, -450, 0, 9090, -37, -37], [4792, 0, -81, 0, 464, 0], [0, -142, 0, -7269, -66, -66], [756, 0, -270, 0, -570, 0], [-627, 0, 26, 0, 1059, 0], [0, 311, 0, -4576, -476, -476], [-5454, 45, 45, 5454, 0, -172], [6655, 299, 299, -6655, 0, 506], [7396, 0, 433, 0, 783, 0], [0, -938, 0, -959, 337, 337], [-2330, 0, -466, 0, -62, 0], [0, 598, 0, -1760, 437, 437], [1136, 872, 872, -1136, 0, -786], [0, -694, 0, 2015, 41, 41], [0, -135, 0, -2933, -745, -745], [9732, 0, 372, 0, -752, 0], [9522, -637, -637, -9522, 0, 183], [-8076, 274, 274, 8076, 0, -945], [0, -204, 0, -14498, -590, -590], [-127, 10, 10, 127, 0, -641], [-39, -476, -476, 39, 0, 376], [-2348, 0, -95, 0, -67, 0], [0, -1196, 0, 1402, -526, -526], [7764, -246, -246, -7764, 0, -502], [16591, 0, 308, 0, 940, 0], [0, 1215, 0, 5147, -446, -446], [-10174, 488, 488, 10174, 0, -226], [16362, 0, 272, 0, -656, 0], [6159, 602, 602, -6159, 0, -1339], [-1336, 0, -547, 0, -1189, 0], [2030, 1325, 1325, -2030, 0, 40], [-8281, 0, 378, 0, -837, 0], [-7118, 114, 114, 7118, 0, 1628], [0, 491, 0, 6090, -1023, -1023], [162, 0, 495, 0, 670, 0], [-127, -143, -143, 127, 0, -1077], [-4525, 0, -896, 0, -102, 0], [0, 1017, 0, 2520, 495, 495], [678, 570, 570, -678, 0, -213], [0, -20, 0, 7791, 641, 641], [-9962, 0, 2056, 0, 961, 0], [0, -64, 0, 26602, -200, -200], [0, 974, 0, -234, 836, 836], [-6587, 0, 35, 0, -572, 0], [0, 879, 0, 11659, -757, -757], [3601, 1548, 1548, -3601, 0, -1401], [10850, 0, 724, 0, 680, 0], [0, 815, 0, -799, -460, -460], [322, 0, -1944, 0, -936, 0], [0, 535, 0, -5446, -1434, -1434], [3051, 0, -1328, 0, 994, 0], [-5874, -1354, -1354, 5874, 0, 480], [2388, 472, 472, -2388, 0, 134], [-12158, 0, -1328, 0, -1257, 0], [8996, 1039, 1039, -8996, 0, -2284], [-3812, 0, 584, 0, 626, 0], [0, -572, 0, 33636, -671, -671], [13968, -144, -144, -13968, 0, 200], [-4080, 0, 62, 0, 139, 0], [-8609, 1173, 1173, 8609, 0, 111], [5820, 0, 1572, 0, 2328, 0], [-873, 0, -563, 0, -1505, 0], [-4760, 346, 346, 4760, 0, 558], [0, -312, 0, 26756, 658, 658], [0, -620, 0, -20586, 1236, 1236], [0, 288, 0, 17586, 782, 782], [11778, 0, 843, 0, 820, 0], [3945, 0, -1100, 0, 279, 0], [0, -998, 0, 610, -622, -622], [-5101, -762, -762, 5101, 0, -1497], [-1326, 0, 807, 0, 2480, 0], [0, -106, 0, -13749, 517, 517], [0, -987, 0, -1528, 502, 502], [-705, 0, 560, 0, -644, 0], [-16950, 1212, 1212, 16950, 0, -381], [0, 1566, 0, 6466, 254, 254], [15118, 0, 1639, 0, 1180, 0], [-23263, 0, -707, 0, 1324, 0], [0, 212, 0, 5050, -1703, -1703], [4546, -1668, -1668, -4546, 0, -28], [-13430, 451, 451, 13430, 0, -426], [-4439, 401, 401, 4439, 0, 2722], [-12947, -2347, -2347, 12947, 0, 1467]]
